#!/usr/bin/env python

# So we can have user input on python2 and 3.
try:
    input = raw_input  # only works for py2
except:
    pass  # so in py3 nothing is done

# Offsets for chord names.
keys = {
    'B#': 0, 'C' : 0,
    'C#': 1, 'Db': 1,
    'D' : 2,
    'D#': 3, 'Eb': 3,
    'E' : 4, 'Fb': 4,
    'E#': 5, 'F' : 5,
    'F#': 6, 'Gb': 6,
    'G' : 7,
    'G#': 8, 'Ab': 8,
    'A' : 9,
    'A#': 10, 'Bb':10,
    'B' : 11, 'Cb':11 }
notes = {
     0: 'C',
     1: 'Db',
     2: 'D',
     3: 'Eb',
     4: 'E',
     5: 'F',
     6: 'Gb',
     7: 'G',
     8: 'Ab',
     9: 'A',
     10: 'Bb',
     11: 'B' }

def normalize(chord):
    """ Convert a list of note values to a set with range 0..11 """
    return set( [a % 12 for a in chord] )

def nl2s(s):
    """ Convert a list of note values (list or set) to a string. """

    s = list(s)  ## works for list and set.
    s = [notes[x] for x in s]
    return ' '.join(s)
    

# Create a dictionary of all the unique chord buttons on the
# the box. Chord buttons all have 3 notes:
#    Major - root, 3rd and 5th
#    Minor - root, diminished 3 and 5th
#    Seventh - root, 3rd and dom. 7. Note: no 5th.
#    Diminished - root, diminished 3 and diminished dom. 7th. No 5th.
# We store a full name as a tuple. Notes are integer values.
buttons = {}
for n in range(0,12):
    buttons[(notes[n], "-M")] = normalize([n, n+4, n+7])
    buttons[(notes[n], "-m")] = normalize([n, n+3, n+7])
    buttons[(notes[n], "-7")] = normalize([n, n+4, n+10])
    buttons[(notes[n], "-d")] = normalize([n, n+3, n+9])

# A list of all known chords. Each entry is the chord type (ie, "m", "M", etc)
# as the key with a tuple of the name and a description. This is lifted
# from MMA - Musical MIDI Accompaniment available at
#   http://mellowood.ca/mma/index.html

chords = {
   '7#5b9': ( (0, 4, 8, 10, 13),  "An augmented chord (raised 5th) with a dominant 7th and flat 9th."),
   'm(sus9)': ( (0, 3, 7, 14),  "Minor triad plus 9th (no 7th)."),
   '7sus': ( (0, 5, 7, 10),  "7th with suspended 4th, dominant 7th with 3rd raised half tone."),
   '7omit3': ( (0, 0, 7, 10),  "7th with unvoiced 3rd."),
   '7b13': ( (0, 4, 7, 10, 20),  "7th (including 5th) plus flat 13th (the 9th and 11th are not voiced)."),
   'aug': ( (0, 4, 8),  "Augmented triad."),
   '13sus4': ( (0, 5, 7, 10, 14, 21),  "7sus, plus 9th and 13th"),
   'm9b5': ( (0, 3, 6, 10, 14),  "Minor triad, flat 5, plus 7th and 9th."),
   '+7b9': ( (0, 4, 8, 10, 13),  "An augmented chord (raised 5th) with a dominant 7th and flat 9th."),
   'aug7b9': ( (0, 4, 8, 10, 13),  "An augmented chord (raised 5th) with a dominant 7th and flat 9th."),
   'm(b5)': ( (0, 3, 6),  "Minor triad with flat 5th (aka dim)."),
   '7b9sus': ( (0, 5, 7, 10, 13),  "7th with suspended 4th and flat 9th."),
   'm7-5': ( (0, 3, 6, 10),  "Minor 7th, flat 5 (aka 1/2 diminished). "),
   'm7b9#11': ( (0, 3, 7, 10, 13, 18),  "Minor 7th plus flat 9th and sharp 11th."),
   '7-9': ( (0, 4, 7, 10, 13),  "7th with flat 9th."),
   'M7(add13)': ( (0, 4, 7, 10, 13, 21),  "7th (including 5th) plus 13th and flat 9th (11th not voiced)."),
   'm#7': ( (0, 3, 7, 11),  "Minor Triad plus Major 7th."),
   'm#5': ( (0, 3, 8),  "Minor triad with augmented 5th."),
   'dim': ( (0, 3, 6, 9),  "A dim7, not a triad!"),
   '7b5': ( (0, 4, 6, 10),  "7th, flat 5."),
   'm7b5': ( (0, 3, 6, 10),  "Minor 7th, flat 5 (aka 1/2 diminished). "),
   'mM7': ( (0, 3, 7, 11),  "Minor Triad plus Major 7th."),
   'm7b9': ( (0, 3, 7, 10, 13),  "Minor 7th with added flat 9th."),
   '13sus': ( (0, 5, 7, 10, 14, 21),  "7sus, plus 9th and 13th"),
   '7b9': ( (0, 4, 7, 10, 13),  "7th with flat 9th."),
   '7(omit3)': ( (0, 0, 7, 10),  "7th with unvoiced 3rd."),
   'dim(b13)': ( (0, 3, 6, 9, 8),  "Diminished seventh, added flat 13th."),
   '7b5b9': ( (0, 4, 6, 10, 13),  "7th with flat 5th and flat 9th."),
   'omit3(add9)': ( (0, 0, 7, 14),  "Triad: root, 5th and 9th."),
   '(add#9)': ( (0, 4, 7, 15),  "Major chord plus sharp 9th (no 7th.)"),
   '9sus': ( (0, 5, 7, 10, 14),  "7sus plus 9th."),
   '(#5)': ( (0, 4, 8),  "Augmented triad."),
   '7b5(add13)': ( (0, 4, 6, 10, 21),  "7th with flat 5 and 13th."),
   '11+': ( (0, 0, 8, 10, 14, 17),  "Augmented 11th (sharp 5)."),
   '+M7': ( (0, 4, 8, 11),  "Major 7th with sharp 5th."),
   'M7': ( (0, 4, 7, 11),  "Major 7th."),
   'M6': ( (0, 4, 7, 9),  "Major tiad with added 6th."),
   'M13#11': ( (0, 4, 7, 11, 18, 21),  "Major 7th plus sharp 11th and 13th (9th not voiced)."),
   '7-5': ( (0, 4, 6, 10),  "7th, flat 5."),
   'addb9': ( (0, 4, 7, 13),  "Major chord plus flat 9th (no 7th.)"),
   'M9': ( (0, 4, 7, 11, 14),  "Major 7th plus 9th."),
   'sus(add9)': ( (0, 5, 7, 14),  "Suspended 4th, major triad with the 3rd raised half tone plus 9th."),
   'M9#11': ( (0, 4, 7, 11, 14, 18),  "Major 9th plus sharp 11th."),
   '+': ( (0, 4, 8),  "Augmented triad."),
   'msus': ( (0, 3, 5, 7),  "Minor suspended 4th, minor triad plus 4th."),
   'm+7': ( (0, 3, 7, 11),  "Minor Triad plus Major 7th."),
   'm+5': ( (0, 3, 8),  "Minor triad with augmented 5th."),
   'sus': ( (0, 5, 7),  "Suspended 4th, major triad with the 3rd raised half tone."),
   '7': ( (0, 4, 7, 10),  "7th."),
   '9sus4': ( (0, 5, 7, 10, 14),  "7sus plus 9th."),
   '7#5#9': ( (0, 4, 8, 10, 15),  "7th with sharp 5th and sharp 9th."),
   '7+': ( (0, 4, 8, 10),  "An augmented chord (raised 5th) with a dominant 7th."),
   '13susb9': ( (0, 5, 7, 10, 13, 21),  "7sus, plus flat 9th and 13th"),
   'm+7b9': ( (0, 3, 8, 10, 13),  "Augmented minor 7 plus flat 9th."),
   '7+9': ( (0, 4, 7, 10, 15),  "7th with sharp 9th."),
   'min(maj7)': ( (0, 3, 7, 11),  "Minor Triad plus Major 7th."),
   'm7(b9)': ( (0, 3, 7, 10, 13),  "Minor 7th with added flat 9th."),
   '13#11': ( (0, 4, 7, 10, 18, 21),  "7th plus sharp 11th and 13th (9th not voiced)."),
   'sus(add#9)': ( (0, 5, 7, 15),  "Suspended 4th, major triad with the 3rd raised half tone plus sharp 9th."),
   '7susb9': ( (0, 5, 7, 10, 13),  "7th with suspended 4th and flat 9th."),
   '13b9': ( (0, 4, 7, 10, 13, 21),  "7th (including 5th) plus 13th and flat 9th (11th not voiced)."),
   '7#5': ( (0, 4, 8, 10),  "An augmented chord (raised 5th) with a dominant 7th."),
   'M7-5': ( (0, 4, 6, 11),  "Major 7th with a flat 5th."),
   '7#9': ( (0, 4, 7, 10, 15),  "7th with sharp 9th."),
   '(add9)': ( (0, 4, 7, 14),  "Major chord plus 9th (no 7th.)"),
   'mM7(add9)': ( (0, 3, 7, 11, 14),  "Minor Triad plus Major 7th and 9th."),
   'add#9': ( (0, 4, 7, 15),  "Major chord plus sharp 9th (no 7th.)"),
   'add9': ( (0, 4, 7, 14),  "Major chord plus 9th (no 7th.)"),
   'm6(add9)': ( (0, 3, 7, 14, 21),  "Minor 6th with added 9th."),
   '13#9': ( (0, 4, 7, 10, 15, 21),  "7th (including 5th) plus 13th and sharp 9th (11th not voiced)."),
   'm7(add13)': ( (0, 3, 7, 10, 21),  "Minor 7th  plus 13th."),
   '+9': ( (0, 4, 8, 10, 14),  "7th plus 9th with sharp 5th (same as aug9)."),
   '11#5': ( (0, 0, 8, 10, 14, 17),  "Augmented 11th (sharp 5)."),
   'm7sus4': ( (0, 3, 5, 7, 10),  "Minor suspended 4th, minor triad plus 4th and dominant 7th."),
   'm11': ( (0, 3, 7, 10, 14, 17),  "9th with minor 3rd,  plus 11th."),
   'm13': ( (0, 3, 7, 10, 21),  "Minor 7th (including 5th) plus 13th (9th and 11th not voiced)."),
   '69': ( (0, 4, 7, 14, 21),  "6th with added 9th. This is sometimes notated as a slash chord in the form ``6/9''."),
   '+9M7': ( (0, 4, 8, 11, 14),  "An augmented chord (raised 5th) with a major 7th and 9th."),
   'min#7': ( (0, 3, 7, 11),  "Minor Triad plus Major 7th."),
   'm7sus': ( (0, 3, 5, 7, 10),  "Minor suspended 4th, minor triad plus 4th and dominant 7th."),
   '7+5': ( (0, 4, 8, 10),  "An augmented chord (raised 5th) with a dominant 7th."),
   'dim7(addM7)': ( (0, 3, 6, 9, 11),  "Diminished tirad with added Major 7th."),
   'm+': ( (0, 3, 8),  "Minor triad with augmented 5th."),
   'm7': ( (0, 3, 7, 10),  "Minor 7th (flat 3rd plus dominant 7th)."),
   'm6': ( (0, 3, 7, 9),  "Minor 6th (flat 3rd plus a 6th)."),
   '6': ( (0, 4, 7, 9),  "Major tiad with added 6th."),
   '(addb9)': ( (0, 4, 7, 13),  "Major chord plus flat 9th (no 7th.)"),
   'm9': ( (0, 3, 7, 10, 14),  "Minor triad plus 7th and 9th."),
   '7#9b13': ( (0, 4, 7, 10, 15, 20),  "7th with sharp 9th and flat 13th."),
   '+7#9': ( (0, 4, 8, 10, 15),  "An augmented chord (raised 5th) with a dominant 7th and sharp 9th."),
   'M#11': ( (0, 4, 7, 11, 18),  "Major triad plus sharp 11th."),
   'M7#5': ( (0, 4, 8, 11),  "Major 7th with sharp 5th."),
   '+7b9#11': ( (0, 4, 8, 10, 13, 18),  "Augmented 7th with flat 9th and sharp 11th."),
   'm7(omit5)': ( (0, 3, 10),  "Minor 7th with unvoiced 5th."),
   'm+7#9': ( (0, 3, 8, 10, 15),  "Augmented minor 7 plus sharp 9th."),
   'mb9': ( (0, 3, 7, 13),  "Minor chord plus flat 9th (no 7th.)"),
   'm+7b9#11': ( (0, 3, 8, 10, 13, 18),  "Augmented minor 7th with flat 9th and sharp 11th."),
   'mb5': ( (0, 3, 6),  "Minor triad with flat 5th (aka dim)."),
   '7#11': ( (0, 4, 7, 10, 18),  "7th plus sharp 11th (9th omitted)."),
   'maj13': ( (0, 4, 7, 11, 21),  "Major 7th (including 5th) plus 13th (9th and  11th not voiced)."),
   'm7b5b9': ( (0, 3, 6, 10, 13),  "Minor 7th with flat 5th and flat 9th."),
   '11': ( (0, 0, 7, 10, 14, 17),  "9th chord plus 11th (3rd not voiced)."),
   '13': ( (0, 4, 7, 10, 21),  "7th (including 5th) plus 13th (the 9th and 11th are not voiced)."),
   '9-5': ( (0, 4, 6, 10, 14),  "7th plus 9th with flat 5th."),
   'msus4': ( (0, 3, 5, 7),  "Minor suspended 4th, minor triad plus 4th."),
   '9b5': ( (0, 4, 6, 10, 14),  "7th plus 9th with flat 5th."),
   '9b6': ( (0, 4, 8, 14),  "9th with flat 6 (no 5th or 7th)."),
   'M11': ( (0, 4, 7, 11, 14, 17),  "Major 9th plus 11th."),
   '11b9': ( (0, 4, 7, 10, 13, 17),  "7th chord plus flat 9th and 11th."),
   'M13': ( (0, 4, 7, 11, 21),  "Major 7th (including 5th) plus 13th (9th and  11th not voiced)."),
   '13b5': ( (0, 4, 6, 10, 20),  "7th with flat 5th,  plus 13th (the 9th and 11th are not voiced)."),
   '7b9#11': ( (0, 4, 7, 10, 13, 18),  "7th plus flat 9th and sharp 11th."),
   'm7(add11)': ( (0, 3, 7, 10, 17),  "Minor 7th  plus 11th."),
   'M7#11': ( (0, 4, 7, 11, 18),  "Major 7th plus sharp 11th (9th omitted)."),
   'm(maj7)': ( (0, 3, 7, 11),  "Minor Triad plus Major 7th."),
   '9+': ( (0, 4, 8, 10, 14),  "7th plus 9th with sharp 5th (same as aug9)."),
   'm7omit5': ( (0, 3, 10),  "Minor 7th with unvoiced 5th."),
   'dim3': ( (0, 3, 6),  "Diminished triad (non-standard notation)."),
   'aug9': ( (0, 4, 8, 10, 14),  "7th plus 9th with sharp 5th (same as aug9)."),
   'dim7': ( (0, 3, 6, 9),  "Diminished seventh."),
   'sus(addb9)': ( (0, 5, 7, 13),  "Suspended 4th, major triad with the 3rd raised half tone plus flat 9th."),
   'm9#11': ( (0, 3, 7, 10, 14, 18),  "Minor 7th plus 9th and sharp 11th."),
   'aug7': ( (0, 4, 8, 10),  "An augmented chord (raised 5th) with a dominant 7th."),
   'M7+5': ( (0, 4, 8, 11),  "Major 7th with sharp 5th."),
   'm11b5': ( (0, 3, 6, 10, 14, 17),  "Minor 7th with flat 5th plus 11th."),
   'm7(#9)': ( (0, 3, 7, 10, 15),  "Minor 7th with added sharp 9th."),
   '(b5)': ( (0, 4, 6),  "Major triad with flat 5th."),
   '7#9#11': ( (0, 4, 7, 10, 15, 18),  "7th plus sharp 9th and sharp 11th."),
   '7(add13)': ( (0, 4, 7, 10, 21),  "7th with added 13th."),
   'm(add9)': ( (0, 3, 7, 14),  "Minor triad plus 9th (no 7th)."),
   '5': ( (0, 0, 7, 7),  "Altered Fifth or Power Chord; root and 5th only."),
   'maj9': ( (0, 4, 7, 11, 14),  "Major 7th plus 9th."),
   '9': ( (0, 4, 7, 10, 14),  "7th plus 9th."),
   '+7': ( (0, 4, 8, 10),  "An augmented chord (raised 5th) with a dominant 7th."),
   'maj7': ( (0, 4, 7, 11),  "Major 7th."),
   'aug7#9': ( (0, 4, 8, 10, 15),  "An augmented chord (raised 5th) with a dominant 7th and sharp 9th."),
   '7b5#9': ( (0, 4, 6, 10, 15),  "7th with flat 5th and sharp 9th."),
   'omit3add9': ( (0, 0, 7, 14),  "Triad: root, 5th and 9th."),
   'M7b5': ( (0, 4, 6, 11),  "Major 7th with a flat 5th."),
   'M': ( (0, 4, 7),  "Major triad. This is the default chord type."),
   '9#11': ( (0, 4, 7, 10, 14, 18),  "7th plus 9th and sharp 11th."),
   '7alt': ( (0, 4, 6, 10, 13),  "Uses a 7th flat 5, flat 9. Probably not correct, but works (mostly)."),
   'sus2': ( (0, 2, 7),  "Suspended 2nd, major triad with the major 2nd above the root substituted for 3rd."),
   'sus4': ( (0, 5, 7),  "Suspended 4th, major triad with the 3rd raised half tone."),
   'sus9': ( (0, 5, 7, 10, 14),  "7sus plus 9th."),
   'aug9M7': ( (0, 4, 8, 11, 14),  "An augmented chord (raised 5th) with a major 7th and 9th."),
   '7(6)': ( (0, 4, 7, 9, 10),  "7th with added 6th."),
   'm7#9': ( (0, 3, 7, 10, 15),  "Minor 7th with added sharp 9th."),
   'm7#5': ( (0, 4, 8, 10),  "Minor 7th with sharp 5th."),
   'm69': ( (0, 3, 7, 14, 21),  "Minor 6th with added 9th."),
   'm': ( (0, 3, 7),  "Minor triad."),
   '9#5': ( (0, 4, 8, 10, 14),  "7th plus 9th with sharp 5th (same as aug9)."),
   '6(add9)': ( (0, 4, 7, 14, 21),  "6th with added 9th."),
   '7sus4': ( (0, 5, 7, 10),  "7th with suspended 4th, dominant 7th with 3rd raised half tone."),
   '7sus2': ( (0, 2, 7, 10),  "A sus2 with dominant 7th added."),
   '9+5': ( (0, 4, 8, 10, 14),  "7th plus 9th with sharp 5th (same as aug9)."),
}

print("Accordion Stradella bass alternate chord finder.")
print("(C) Bob van der Poel <bob@mellowood.ca> 2017")
print('')
print("Note: All chord names and pitches are case sensitive!")
print("      Reported matches show flats. You'll have to find the #s on your own.")
print('')
while 1:
    print('')
    pitch = input("Enter pitch of chord (ie. C, Gb, F#, etc.):")
    if pitch not in keys:
        print("The pitch %s is not valid. Please use: %s" %
              (pitch, ', '.join([x for x in sorted(keys)])))
        continue

    chordtype = input("Enter chord type:")
    if chordtype not in chords:
        print("The chordtype %s is not valid. Please use 1 of the following %s:\n%s" %
              (chordtype, len(chords), ', '.join([x for x in sorted(chords)])))
        continue

    searchNotes = chords[chordtype][0]
    searchNotes = [x+keys[pitch] for x in searchNotes]
    searchNotes = normalize(searchNotes)

    print("Chord '%s%s' notes are: %s" % \
        (pitch, chordtype, nl2s(searchNotes)))

    for t in buttons:
        if searchNotes >= buttons[t]:
            butName = "%s%s" % t
            hasNotes = nl2s(buttons[t])
            missNotes = nl2s(searchNotes - buttons[t])
            if not missNotes:
                missNotes = "None"
            print("  Button %s has: %s; missing %s" % (butName, hasNotes, missNotes))
